from .bblock import Doc, Sentences  # noqa: F401
from .bblock.token import Token  # noqa: F401
from .config import set_config, get_config, get_all_configs, describe_config, tokenizer_context  # noqa: F401



import unicodedata

from math import log
import numpy as np
from .util import tr_lower, load_stopwords

IDF_SMOOTH, IDF_PROBABILISTIC = "smooth", "probabilistic"
IDF_METHOD_VALUES = [IDF_SMOOTH, IDF_PROBABILISTIC]


class IDFImpl:
    def __init__(self):
        pass

    def get_idf(self, method=IDF_SMOOTH, drop_stopwords=False, lowercase=False, drop_suffix=False, drop_punct=False,
                **kwargs):

        if method not in IDF_METHOD_VALUES:
            raise ValueError(f"Unknown idf method ({method}). Choose one of {IDF_METHOD_VALUES}")

        v = np.zeros(len(self.vocabulary))

        if lowercase:
            tokens = [tr_lower(t) for t in self.tokens]
        else:
            tokens = self.tokens

        for token in tokens:
            t = self.vocabulary[token]
            if t.is_oov or (drop_stopwords and t.is_stopword) or (drop_suffix and t.is_suffix) or (
                    drop_punct and t.is_punct):
                continue

            if method == IDF_SMOOTH:
                v[t.id] = t.smooth_idf
            else:
                v[t.id] = t.prob_idf

        return v


def word_shape(text):
    if len(text) >= 100:
        return "LONG"
    shape = []
    last = ""
    shape_char = ""
    seq = 0
    for char in text:
        if char.isalpha():
            if char.isupper():
                shape_char = "X"
            else:
                shape_char = "x"
        elif char.isdigit():
            shape_char = "d"
        else:
            shape_char = char
        if shape_char == last:
            seq += 1
        else:
            seq = 0
            last = shape_char
        if seq < 4:
            shape.append(shape_char)
    return "".join(shape)


class Token:
    config = None
    STOPWORDS = None
    cache = {}

    def __init__(self, entry):
        if self in self.cache:
            return

        if isinstance(entry, str):
            self.cache[entry] = self

            self.word = entry
            self.lower_ = tr_lower(self.word)
            self.is_punct = all(unicodedata.category(c).startswith("P") for c in self.word)
            self.is_digit = self.word.isdigit()
            self.is_suffix = self.word.startswith('##')
            self.shape = word_shape(self.word)

            self._entry = None
        else:
            self.cache[entry.word] = self

            self.word = entry.word
            self.lower_ = tr_lower(self.word)
            self.is_punct = all(unicodedata.category(c).startswith("P") for c in self.word)
            self.is_digit = self.word.isdigit()
            self.is_suffix = self.word.startswith('##')
            self.shape = word_shape(self.word)

            self._entry = entry

    @property
    def entry(self):
        if self._entry is None:
            raise ValueError(f"Token is initialized with a str object. Initialized with Vocabulary entry")

        return self._entry

    @property
    def idf(self):
        if Token.config['idf']['method'] == IDF_SMOOTH:
            return self.smooth_idf
        else:
            return self.prob_idf

    @property
    def smooth_idf(self):
        return log(self.entry.vocabulary.document_count / (1 + self.df)) + 1

    @property
    def prob_idf(self):
        return log((self.entry.vocabulary.document_count - self.df) / self.df)

    @property
    def id(self):
        return self.entry.id

    @property
    def is_oov(self):
        return self.id == -1

    @property
    def is_stopword(self):
        if Token.STOPWORDS is None:
            Token.STOPWORDS = set(load_stopwords())

        return self.lower_ in Token.STOPWORDS

    @property
    def df(self):
        return self.entry.df

    @property
    def df_cs(self):
        """case sensitive document frequency"""
        return self.entry.df_cs

    def __str__(self):
        return self.word

    def __repr__(self):
        return self.word



import numpy as np
from scipy.sparse.csgraph import connected_components


def _power_method(transition_matrix, increase_power=True):
    eigenvector = np.ones(len(transition_matrix))

    if len(eigenvector) == 1:
        return eigenvector

    transition = transition_matrix.transpose()

    while True:
        eigenvector_next = np.dot(transition, eigenvector)

        if np.allclose(eigenvector_next, eigenvector):
            return eigenvector_next

        eigenvector = eigenvector_next

        if increase_power:
            transition = np.dot(transition, transition)


def connected_nodes(matrix):
    _, labels = connected_components(matrix)

    groups = []

    for tag in np.unique(labels):
        group = np.where(labels == tag)[0]
        groups.append(group)

    return groups


def create_markov_matrix(weights_matrix):
    n_1, n_2 = weights_matrix.shape
    if n_1 != n_2:
        raise ValueError('\'weights_matrix\' should be square')

    row_sum = weights_matrix.sum(axis=1, keepdims=True)

    return weights_matrix / row_sum


def create_markov_matrix_discrete(weights_matrix, threshold):
    discrete_weights_matrix = np.zeros(weights_matrix.shape)
    ixs = np.where(weights_matrix >= threshold)
    discrete_weights_matrix[ixs] = 1

    return create_markov_matrix(discrete_weights_matrix)


def graph_nodes_clusters(transition_matrix, increase_power=True):
    clusters = connected_nodes(transition_matrix)
    clusters.sort(key=len, reverse=True)

    centroid_scores = []

    for group in clusters:
        t_matrix = transition_matrix[np.ix_(group, group)]
        eigenvector = _power_method(t_matrix, increase_power=increase_power)
        centroid_scores.append(eigenvector / len(group))

    return clusters, centroid_scores


def stationary_distribution(
        transition_matrix,
        increase_power=True,
        normalized=True,
):
    n_1, n_2 = transition_matrix.shape
    if n_1 != n_2:
        raise ValueError('\'transition_matrix\' should be square')

    distribution = np.zeros(n_1)

    grouped_indices = connected_nodes(transition_matrix)

    for group in grouped_indices:
        t_matrix = transition_matrix[np.ix_(group, group)]
        eigenvector = _power_method(t_matrix, increase_power=increase_power)
        distribution[group] = eigenvector

    if normalized:
        distribution /= n_1

    return distribution


def degree_centrality_scores(
        similarity_matrix,
        threshold=None,
        increase_power=True,
):
    if not (
            threshold is None
            or isinstance(threshold, float)
            and 0 <= threshold < 1
    ):
        raise ValueError(
            '\'threshold\' should be a floating-point number '
            'from the interval [0, 1) or None',
        )

    if threshold is None:
        markov_matrix = create_markov_matrix(similarity_matrix)

    else:
        markov_matrix = create_markov_matrix_discrete(
            similarity_matrix,
            threshold,
        )

    scores = stationary_distribution(
        markov_matrix,
        increase_power=increase_power,
        normalized=False,
    )

    return scores



from abc import ABCMeta, abstractmethod
from typing import List

import re
import nltk

alphabets = "([A-Za-z])"
prefixes = "(Mr|St|Mrs|Ms|Dr)[.]"
suffixes = "(Inc|Ltd|Jr|Sr|Co)"
starters = r"(Mr|Mrs|Ms|Dr|He\s|She\s|It\s|They\s|Their\s|Our\s|We\s|But\s|However\s|That\s|This\s|Wherever)"
acronyms = "([A-Z][.][A-Z][.](?:[A-Z][.])?)"
websites = "[.](com|net|org|io|gov)"


class SentencesTokenizer(object, metaclass=ABCMeta):
    def __init__(self):
        pass

    def __call__(self, doc: str) -> List[str]:
        return self._split(doc)

    @abstractmethod
    def _split(self, text):
        pass


class RegexpSentenceTokenizer(SentencesTokenizer):

    def _split(self, text: str) -> List[str]:
        text = " " + text + "  "
        text = text.replace(r"\n", " ")
        text = re.sub(prefixes, "\\1<prd>", text)
        text = re.sub(websites, "<prd>\\1", text)

        if "Ph.D" in text:
            text = text.replace("Ph.D.", "Ph<prd>D<prd>")

        text = re.sub(r"\s" + alphabets + "[.] ", " \\1<prd> ", text)
        text = re.sub(acronyms + " " + starters, "\\1<stop> \\2", text)
        text = re.sub(alphabets + "[.]" + alphabets + "[.]" + alphabets + "[.]", "\\1<prd>\\2<prd>\\3<prd>", text)
        text = re.sub(alphabets + "[.]" + alphabets + "[.]", "\\1<prd>\\2<prd>", text)
        text = re.sub(" " + suffixes + "[.] " + starters, " \\1<stop> \\2", text)
        text = re.sub(" " + suffixes + "[.]", " \\1<prd>", text)
        text = re.sub(" " + alphabets + "[.]", " \\1<prd>", text)

        if "”" in text:
            text = text.replace(".”", "”.")

        if "\"" in text:
            text = text.replace(".\"", "\".")

        if "!" in text:
            text = text.replace("!\"", "\"!")

        if "?" in text:
            text = text.replace("?\"", "\"?")

        text = text.replace(".", ".<stop>")
        text = text.replace("?", "?<stop>")
        text = text.replace("!", "!<stop>")
        text = text.replace("<prd>", ".")
        sentences = text.split("<stop>")
        sentences = sentences[:-1]
        sentences = [s.strip() for s in sentences if len(s) > 1]

        return sentences


class NLTKPunctTokenizer(SentencesTokenizer):
    def __init__(self):
        super().__init__()

        self.sent_detector = nltk.data.load('tokenizers/punkt/turkish.pickle')

    def _split(self, text: str) -> List[str]:
        return self.sent_detector.tokenize(text)



from typing import List
from loguru import logger
from math import ceil

from rich.progress import track

from sklearn.metrics import ndcg_score
import numpy as np

from sadedegel.config import tokenizer_context
from sadedegel.summarize import BandSummarizer, RandomSummarizer
from sadedegel.dataset import load_annotated_corpus

logger.disable("sadedegel")

import wandb


# wandb.login()


def evaluate_summarizer(summarizer, relevance, doc_sg):
    score_10, score_50, score_80 = [], [], []

    for i, (y_true, d) in enumerate(zip(relevance, doc_sg)):
        y_pred = [summarizer.predict([s for s in d])]

        score_10.append(ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.1)))
        score_50.append(ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.5)))
        score_80.append(ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.8)))

    return np.array(score_10).mean(), np.array(score_50).mean(), np.array(score_80).mean()


def run_rand(docs):
    wandb.init(project='sadedegel')

    from sadedegel import Doc
    config = wandb.config
    config.method = "rand"

    relevance = [[doc['relevance']] for doc in docs]

    doc_sg = [Doc.from_sentences(doc['sentences']) for doc in track(docs)]

    summarizer = RandomSummarizer()

    score_10, score_50, score_80 = evaluate_summarizer(summarizer, relevance, doc_sg)

    wandb.log(dict(score_10=score_10, score_50=score_50,
                   score_80=score_50))


def run_band(docs, tokenizers: List[int], k_list: List[int]):
    wandb.init(project='sadedegel')

    config = wandb.config
    config.method = "band"

    relevance = [[doc['relevance']] for doc in docs]

    for tokenizer in tokenizers:
        config.tokenizer = tokenizer

        with tokenizer_context(tokenizer) as Doc:

            doc_sg = [Doc.from_sentences(doc['sentences']) for doc in track(docs)]

            for k in k_list:
                config.k = k
                summarizer = BandSummarizer(k)

                score_10, score_50, score_80 = evaluate_summarizer(summarizer, relevance, doc_sg)

                wandb.log(dict(score_10=score_10, score_50=score_50,
                               score_80=score_50))


if __name__ == '__main__':
    anno = load_annotated_corpus(False)

    run_band(anno, tokenizers=['simple', 'bert'], k_list=[2, 3, 6, 8])
    run_rand(anno)



import glob
from os.path import dirname, join, expanduser, basename, splitext
from loguru import logger
import numpy as np
import re
from enum import Enum
from .util import safe_json_load, safe_read
from pathlib import Path


class CorpusTypeEnum(Enum):
    RAW = ('raw', '*.txt')
    SENTENCE = ('sents', '*.json')
    ANNOTATED = ('annotated', '*.json')

    def __init__(self, dir, pattern):
        self.dir = dir
        self.pattern = pattern


def file_paths(corpus_type: CorpusTypeEnum = CorpusTypeEnum.RAW, noext=False, use_basename=False, base_path=None):
    if base_path is None:
        base_path = dirname(__file__)

    if type(corpus_type) == CorpusTypeEnum:
        search_pattern = Path(base_path).expanduser() / corpus_type.dir

        logger.debug(f"Search pattern for {corpus_type}: {search_pattern}")
        files = search_pattern.glob(corpus_type.pattern)
    else:
        raise ValueError(f"Ensure that corpus_type is a one of raw, sentence or annotated")

    if use_basename:
        if noext:
            return sorted([splitext(basename(fn))[0] for fn in files])
        else:
            return sorted([basename(fn) for fn in files])
    else:
        return sorted(files)


def cleaner(doc: str):
    return re.sub(r'\d+\s+[a-zA-ZŞşğĞüÜıİ]+\s+\d{4}\s+PAYLAŞ\s+yorum\s+yaz(\s+a)?', '', doc, flags=re.I)


def load_raw_corpus(return_iter: bool = True, base_path=None, clean=True):
    """Load corpus of sample news tokenized into sentences.

    Examples
    --------
    >>> from sadedegel.dataset import load_raw_corpus
    >>> sents = load_raw_corpus(return_iter=False)
    >>> type(sents[0])
    <class 'str'>

    """
    if base_path is None:
        base_path = dirname(__file__)

    search_pattern = join(expanduser(base_path), 'raw', '*.txt')

    logger.debug("Search path {}".format(search_pattern))

    files = sorted(glob.glob(search_pattern))

    if return_iter:
        if clean:
            return (cleaner(safe_read(file)) for file in files)
        else:
            return (safe_read(file) for file in files)
    else:
        if clean:
            return [cleaner(safe_read(file)) for file in files]
        else:
            return [safe_read(file) for file in files]


def load_sentence_corpus(return_iter: bool = True, base_path=None):
    """Load corpus of sample news tokenized into sentences.

    Examples
    --------
    >>> from sadedegel.dataset import load_sentence_corpus
    >>> sents = load_sentence_corpus(return_iter=False)
    >>> type(sents[0])
    <class 'dict'>

    >>> len(sents[0]['sentences'])
    62
    """
    if base_path is None:
        base_path = dirname(__file__)

    search_pattern = join(expanduser(base_path), 'sents', '*.json')

    logger.debug("Search path {}".format(search_pattern))

    files = sorted(glob.glob(search_pattern))

    if return_iter:
        return map(safe_json_load, files)
    else:
        return [safe_json_load(file) for file in files]


def load_annotated_corpus(return_iter: bool = True, base_path=None):
    """Load corpus of sample news tokenized into sentences and scored based on human annotation"""

    files = file_paths(CorpusTypeEnum.ANNOTATED, base_path)

    def to_dict(d):
        return dict(sentences=[s['content'] for s in d['sentences']],
                    relevance=np.array([s['deletedInRound'] for s in d['sentences']]))

    if return_iter:
        return map(to_dict,
                   map(safe_json_load, files))
    else:
        return [to_dict(safe_json_load(file)) for file in files]



import os.path
from pathlib import Path
import sys
from itertools import tee

from smart_open import open

import click

import boto3

from loguru import logger

from ._core import load_tokenization_tokenized, load_tokenization_raw, CORPUS_SIZE, tarballs

from rich.console import Console
from rich.table import Table

console = Console()

logger.disable("sadedegel")


@click.group(help="Tokenization Dataset Commandline")
def cli():
    pass


@cli.command()
@click.option("--access-key", help="Access Key ID to download dataset.", prompt=True,
              default=lambda: os.environ.get('sadedegel_access_key', ''))
@click.option("--secret-key", help="Secret Key ID to download dataset.", prompt=True,
              default=lambda: os.environ.get('sadedegel_secret_key', ''))
@click.option("--data_home", '-d', help="Data home directory", default="~/.sadedegel_data")
def download(access_key, secret_key, data_home):
    """Download tokenization corpus from cloud with your key."""

    data_home = Path(os.path.expanduser(data_home)) / 'tscorpus'
    console.print(f"Data directory for tokenization data {data_home}")

    transport_params = {
        'session': boto3.Session(aws_access_key_id=access_key,
                                 aws_secret_access_key=secret_key),
        'resource_kwargs': {
            'endpoint_url': 'https://storage.googleapis.com',
        }
    }

    for fmt in ['raw', 'tokenized']:
        (data_home / fmt).mkdir(parents=True, exist_ok=True)

        with console.status(f"[bold green]Downloading {fmt}...") as status:
            for tarball in tarballs:
                url = f"s3://sadedegel/dataset/tscorpus/{fmt}/{tarball}"

                with open(url, 'rb', transport_params=transport_params) as fp, open(data_home / fmt / tarball,
                                                                                    "wb") as wp:
                    wp.write(fp.read())

                console.log(f"{fmt}/{tarball} complete.")


@cli.command()
def validate():
    raw = load_tokenization_raw()
    tok = load_tokenization_tokenized()

    with console.status("[bold yellow]Validating tscorpus"):

        raw_clone, raw = tee(raw, 2)
        tok_clone, tok = tee(tok, 2)

        n_doc_raw = sum(1 for _ in raw_clone)
        n_doc_tok = sum(1 for _ in tok_clone)

        if n_doc_raw != n_doc_tok:
            console.log("Cardinality check [red]FAILED[/red]")
            console.log(f"|TsCorpus (raw)|: {n_doc_raw}")
            console.log(f"|TsCorpus (tokenized)|: {n_doc_tok}")

            sys.exit(1)
        else:
            if n_doc_raw == CORPUS_SIZE:
                console.log("Cardinality check [yellow]DONE[/yellow]")
                n_document = CORPUS_SIZE
            else:
                console.log("Cardinality check [red]FAILED[/red]")
                console.log(f"|TsCorpus (raw)| : {n_doc_raw}")
                sys.exit(1)

        count = 0
        for i, (d1, d2) in enumerate(zip(raw, tok)):
            if d1['id'] != d2['id']:
                console.log("Document order check [red]FAILED[/red]")
                console.log(f"{i}th document (0-based index) raw#:{d1['id']}, tokenized#:{d2['id']}")

                sys.exit(1)

            count += 1
        console.log("Document order check [yellow]DONE[/yellow]")

        if count != n_document:
            console.log(f"[red]Corpus cardinality is {count}/{n_document}. 302936 is expected.")
            sys.exit(1)
        else:

            console.log("TsCorpus is [green]VALID[/green]")


if __name__ == "__main__":
    cli()



from typing import List
import warnings
import numpy as np
from sklearn.metrics.pairwise import cosine_similarity
import networkx as nx
from ._base import ExtractiveSummarizer
from ..bblock import Sentences
from ..config import tokenizer_context
from lexrank import LexRank
from ..dataset import load_raw_corpus
from ..bblock.util import load_stopwords
from ..about import __version__

from .util.power_method import degree_centrality_scores
from ..bblock.doc import TF_RAW, TF_FREQ, TF_BINARY, TF_LOG_NORM, TF_DOUBLE_NORM
from ..bblock.token import IDF_PROBABILISTIC, IDF_SMOOTH


class TextRank(ExtractiveSummarizer):
    """Assign a higher importance score to longer sentences.

    mode : {'token', 'char'}, default='token'
        Whether the distance should be defined by total number of characters or
        total number of tokens in a sentence.

    normalize : bool, optional (default=True)
        If ``False``, return a raw score vector.
        Otherwise, return a score vector that adds up to 1.
    """

    tags = ExtractiveSummarizer.tags + ['ml', 'rank', 'graph']

    def __init__(self, input_type="bert", alpha=0.5, normalize=True):
        super().__init__(normalize)

        if input_type not in ['bert']:
            raise ValueError(f"mode should be one of 'bert'")

        self.input_type = input_type
        self.alpha = alpha

    def _predict(self, sentences: List[Sentences]) -> np.ndarray:
        if len(sentences) == 0:
            raise ValueError(f"Ensure that document contains a few sentences for summarization")

        if sentences[0].tokenizer.__name__ != "BertTokenizer":
            with tokenizer_context('bert', warning=True) as Doc2:
                doc = Doc2(sentences[0].document.raw)
        else:
            doc = sentences[0].document

        vectors = doc.bert_embeddings

        similarity_matrix = np.zeros((len(vectors), len(vectors)))

        for i in range(len(vectors)):
            for j in range(len(vectors)):
                if i == j:
                    continue

                similarity_matrix[i][j] = cosine_similarity(vectors[i].reshape(1, -1), vectors[j].reshape(1, -1))[
                    0, 0]

        nx_graph = nx.from_numpy_array(similarity_matrix)
        scores = nx.pagerank(nx_graph, alpha=self.alpha)

        return np.array([scores[i] for i in range(len(scores))])


class LexRankSummarizer(ExtractiveSummarizer):
    """
        Unsupervised summarizer which just like Rouge1 summarizer
        uses sentence's similarity to other sentences.
        Github: https://github.com/crabcamp/lexrank/
    """

    tags = ExtractiveSummarizer.tags + ['ml', 'rank', 'graph']

    def __init__(self, normalize=True):
        super().__init__(normalize)

        stopwords = set(load_stopwords())
        corpus = load_raw_corpus(return_iter=False)
        self.lxr = LexRank(corpus, stopwords=stopwords)

    def _predict(self, sentences: List[Sentences]) -> np.ndarray:
        sentences = [str(s) for s in sentences]
        scores = self.lxr.rank_sentences(
            sentences,
            threshold=None,
            fast_power_method=False,
        )

        return np.array(scores)


class LexRankPureSummarizer(ExtractiveSummarizer):
    """
        Unsupervised summarizer which just like Rouge1 summarizer
        uses sentence's similarity to other sentences.
    """

    tags = ExtractiveSummarizer.tags + ['ml', 'rank', 'graph9']

    def __init__(self, normalize=True, tf_method=None, idf_method=None, threshold=.03, fast_power_method=True,
                 **kwargs):
        super().__init__(normalize)

        if tuple(map(int, __version__.split('.'))) < (0, 18):
            warnings.warn(
                ("LexRankPureSummarizer is a pure sadedegel based implementation of lexrank."
                 "It is deprecated as LexRankPureSummarizer has a better performance than original LexRankSummarizer,"
                 "so we will rename this summarizer as LexRankSummarizer and drop lexrank dependency by 0.18.")
                , DeprecationWarning,
                stacklevel=2)

        self.tf_method = tf_method
        self.idf_method = idf_method

        if not (threshold is None or isinstance(threshold, float) and 0 <= threshold < 1):
            raise ValueError("'threshold' should be a floating-point number from the interval [0, 1) or None")

        self.threshold = threshold
        self.fast_power_method = fast_power_method

    def _predict(self, sentences: List[Sentences]) -> np.ndarray:
        similarity_matrix = np.zeros((len(sentences), len(sentences)))

        for i in range(len(sentences)):
            for j in range(len(sentences)):
                if i == j:
                    continue

                s1 = sentences[i]
                s2 = sentences[j]

                similarity_matrix[i][j] = \
                    cosine_similarity(s1.get_tfidf(self.tf_method, self.idf_method, drop_stopwords=True, lowercase=True,
                                                   drop_suffix=True,
                                                   drop_punct=True).reshape(1, -1),
                                      s2.get_tfidf(self.tf_method, self.idf_method, drop_stopwords=True, lowercase=True,
                                                   drop_suffix=True,
                                                   drop_punct=True).reshape(1,
                                                                            -1))[
                        0, 0]

        scores = degree_centrality_scores(
            similarity_matrix,
            threshold=self.threshold,
            increase_power=self.fast_power_method,
        )

        return scores



from collections import defaultdict
from math import ceil
from typing import List
from loguru import logger

import click
import time
import warnings
from sadedegel.about import __version__

if tuple(map(int, __version__.split('.'))) < (0, 18):
    warnings.warn(
        "tabulate package is deprecated and will be removed by 0.18. "
        , DeprecationWarning,
        stacklevel=2)

    from tabulate import tabulate
else:
    raise Exception("Drop tabulate dependency")

import warnings
import numpy as np  # type: ignore
from sklearn.metrics import ndcg_score  # type: ignore

from sadedegel.dataset import load_annotated_corpus
from sadedegel.summarize import RandomSummarizer, PositionSummarizer, Rouge1Summarizer, KMeansSummarizer, \
    AutoKMeansSummarizer, \
    DecomposedKMeansSummarizer, LengthSummarizer, TextRank, LexRankSummarizer, LexRankPureSummarizer, TFIDFSummarizer, \
    BandSummarizer
from sadedegel import Sentences
from sadedegel.bblock import DocBuilder
from sadedegel import tokenizer_context
from sadedegel.config import config_context
from sadedegel.bblock.doc import TF_METHOD_VALUES, IDF_METHOD_VALUES

logger.disable("sadedegel")

SUMMARIZERS = [('Random Summarizer', RandomSummarizer()), ('FirstK Summarizer', PositionSummarizer()),
               ('LastK Summarizer', PositionSummarizer('last')), ('Rouge1 Summarizer (f1)', Rouge1Summarizer()),
               ("Band(k=2) Summarizer", BandSummarizer(2)),
               ("Band(k=3) Summarizer", BandSummarizer(3)),
               ("Band(k=6) Summarizer", BandSummarizer(6)),
               ('Rouge1 Summarizer (precision)', Rouge1Summarizer('precision')),
               ('Rouge1 Summarizer (recall)', Rouge1Summarizer('recall')),
               ('Length Summarizer (char)', LengthSummarizer('token')),
               ('Length Summarizer (token)', LengthSummarizer('char')),
               ('KMeans Summarizer', KMeansSummarizer()),
               ('AutoKMeans Summarizer', AutoKMeansSummarizer()),
               ('DecomposedKMeans Summarizer', DecomposedKMeansSummarizer()),
               ("TextRank(0.05) Summarizer (BERT)", TextRank(alpha=0.05)),
               ("TextRank(0.15) Summarizer (BERT)", TextRank(alpha=0.15)),
               ("TextRank(0.30) Summarizer (BERT)", TextRank(alpha=0.30)),
               ("TextRank(0.5) Summarizer (BERT)", TextRank(alpha=0.5)),
               ("TextRank(0.6) Summarizer (BERT)", TextRank(alpha=0.6)),
               ("TextRank(0.7) Summarizer (BERT)", TextRank(alpha=0.7)),
               ("TextRank(0.85) Summarizer (BERT)", TextRank(alpha=0.85)),
               ("TextRank(0.9) Summarizer (BERT)", TextRank(alpha=0.9)),
               ("TextRank(0.95) Summarizer (BERT)", TextRank(alpha=0.95)),
               ("LexRank Summarizer", LexRankSummarizer()),
               ("LexRankPure Summarizer", LexRankPureSummarizer(tf_method="raw", idf_method="probabilistic"))]


def to_sentence_list(sents: List[str]) -> List[Sentences]:
    l: List[Sentences] = []

    for i, sent in enumerate(sents):
        l.append(Sentences(i, sent, l))

    return l


@click.group(help="SadedeGel summarizer commandline")
def cli():
    pass


def dot_progress(i, length, t0):
    n = ceil(length * 0.05)

    if i == length - 1:
        click.echo(f". {(time.time() - t0):.2f} sec", nl=True, color="yellow")
    elif i % n == 0 and i != 0:
        click.echo(".", nl=False, color="yellow")


@cli.command()
@click.option("-f", "--table-format", default="github")
@click.option("-t", "--tag", default=["extractive"], multiple=True)
@click.option("-d", "--debug", default=False)
def evaluate(table_format, tag, debug):
    """Evaluate all summarizers in sadedeGel"""
    if not debug:
        warnings.filterwarnings("ignore")

    anno = load_annotated_corpus(False)
    relevance = [[doc['relevance']] for doc in anno]

    summarizers = [summ for summ in SUMMARIZERS if any(_tag in summ[1] for _tag in tag)]

    scores = defaultdict(list)

    for word_tokenizer in ['simple', 'bert']:
        click.echo("Word Tokenizer: " + click.style(f"{word_tokenizer}", fg="blue"))

        with tokenizer_context(word_tokenizer) as Doc2:
            docs = [Doc2.from_sentences(doc['sentences']) for doc in
                    anno]
            for name, summarizer in summarizers:
                t0 = time.time()
                click.echo(click.style(f"    {name} ", fg="magenta"), nl=False)
                # skip simple tokenizer for clustering models
                if ("cluster" in summarizer or "rank" in summarizer or name == "TFIDF Summarizer") and \
                        word_tokenizer == "simple":
                    click.echo(click.style("SKIP", fg="yellow"))
                    continue

                for i, (y_true, d) in enumerate(zip(relevance, docs)):
                    dot_progress(i, len(relevance), t0)

                    y_pred = [summarizer.predict(d)]

                    score_10 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.1))
                    score_50 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.5))
                    score_80 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.8))

                    scores[f"{name} - {word_tokenizer}"].append((score_10, score_50, score_80))

    name, summarizer = "TFIDF Summarizer", TFIDFSummarizer()

    if any(_tag in summarizer for _tag in tag):
        for tf in TF_METHOD_VALUES:
            for idf in IDF_METHOD_VALUES:
                if tf == "double_norm":
                    for k in [0.25, 0.5, 0.75]:
                        with config_context(tokenizer="bert", tf__method=tf, idf__method=idf,
                                            tf__double_norm_k=k) as Doc2:
                            t0 = time.time()
                            table_key = f"{name} (tf={tf}, double_norm_k={k}, idf={idf}, tokenizer=bert)"
                            click.echo(click.style(f"    {table_key}", fg="magenta"), nl=False)

                            docs = [Doc2.from_sentences(doc['sentences']) for doc in
                                    anno]

                            for i, (y_true, d) in enumerate(zip(relevance, docs)):
                                dot_progress(i, len(relevance), t0)

                                y_pred = [summarizer.predict(d)]

                                score_10 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.1))
                                score_50 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.5))
                                score_80 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.8))

                                scores[table_key].append((score_10, score_50, score_80))
                else:
                    with config_context(tokenizer="bert", tf__method=tf, idf__method=idf) as Doc2:
                        t0 = time.time()
                        table_key = f"{name} (tf={tf}, idf={idf}, tokenizer=bert)"
                        click.echo(click.style(f"    {table_key}", fg="magenta"), nl=False)

                        docs = [Doc2.from_sentences(doc['sentences']) for doc in
                                anno]

                        for i, (y_true, d) in enumerate(zip(relevance, docs)):
                            dot_progress(i, len(relevance), t0)

                            y_pred = [summarizer.predict(d)]

                            score_10 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.1))
                            score_50 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.5))
                            score_80 = ndcg_score(y_true, y_pred, k=ceil(len(d) * 0.8))

                            scores[table_key].append((score_10, score_50, score_80))

    table = [[algo, np.array([s[0] for s in scores]).mean(), np.array([s[1] for s in scores]).mean(),
              np.array([s[2] for s in scores]).mean()] for
             algo, scores in scores.items()]

    # TODO: Sample weight of instances.
    print(
        tabulate(table, headers=['Method & Tokenizer', 'ndcg(k=0.1)', 'ndcg(k=0.5)', 'ndcg(k=0.8)'],
                 tablefmt=table_format,
                 floatfmt=".4f"))

    if debug:
        click.echo(np.array(table).shape)


if __name__ == '__main__':
    cli()



from ._core import check_and_display

from ._core import load_tscorpus_raw, load_classification_raw, load_tokenization_raw, load_tokenization_tokenized, \
    fail_with_corpus_validation, \
    CORPUS_SIZE, CATEGORIES



from os.path import expanduser, getsize
from pathlib import Path
import json
from typing import Iterator, Tuple
import glob
import click

__download_message__ = """Ensure that you have properly downloaded extended corpus using
         
            python -m sadedegel.dataset.extended download --access-key xxx --secret-key xxxx
            
        Unfortunately due to data licensing issues we could not share data publicly. 
        Get in touch with sadedegel team to obtain a download key.
        
        """


def check_directory_structure(path: str) -> bool:
    if not Path(expanduser(path)).exists():
        click.secho(f"{path} not found.\n", fg="red")
        click.secho(__download_message__, fg="red")

        return False
    elif not (Path(expanduser(path)) / 'extended' / 'raw').exists():
        click.secho(f"raw directory in {path} not found.\n", fg="red")
        click.secho(__download_message__, fg="red")

        return False
    elif not (Path(expanduser(path)) / 'extended' / 'sents').exists():
        click.secho(f"sents directory in {path} not found.\n", fg="red")
        click.secho(__download_message__, fg="red")

        return False
    else:
        return True


def raw_stats(data_home: str) -> Tuple[int, int]:
    n, sz = 0, 0
    for f in glob.glob(str((Path(expanduser(data_home)) / 'extended' / 'raw' / '*' / '*.txt').absolute())):
        n += 1
        sz += getsize(f)

    return n, sz


def sents_stats(data_home: str) -> Tuple[int, int]:
    n, sz = 0, 0
    for f in glob.glob(str((Path(expanduser(data_home)) / 'extended' / 'sents' / '*' / '*.json').absolute())):
        n += 1
        sz += getsize(f)

    return n, sz


def load_extended_metadata(data_home="~/.sadedegel_data"):
    if check_directory_structure(data_home):

        raw_count, raw_bytes = raw_stats(data_home)
        sents_count, sents_bytes = sents_stats(data_home)

        return dict(count=dict(raw=raw_count, sents=sents_count), byte=dict(raw=raw_bytes, sents=sents_bytes))
    else:
        return None


def load_extended_raw_corpus(data_home="~/.sadedegel_data") -> Iterator[str]:
    if check_directory_structure(data_home):
        for f in glob.glob(str((Path(expanduser(data_home)) / 'extended' / 'raw' / '*' / '*.txt').absolute())):
            with open(f) as fp:
                yield fp.read()
    else:
        return None


def load_extended_sents_corpus(data_home="~/.sadedegel_data") -> Iterator[dict]:
    if check_directory_structure(data_home):
        for f in glob.glob(str((Path(expanduser(data_home)) / 'extended' / 'sents' / '*' / '*.json').absolute())):
            with open(f) as fp:
                yield json.load(fp)
    else:
        return None



from .baseline import RandomSummarizer, PositionSummarizer, LengthSummarizer, BandSummarizer  # noqa: F401
from .rouge import Rouge1Summarizer  # noqa: F401
from .cluster import KMeansSummarizer, AutoKMeansSummarizer, DecomposedKMeansSummarizer  # noqa: F401
from .rank import TextRank, LexRankSummarizer, LexRankPureSummarizer  # noqa: F401
from .tf_idf import TFIDFSummarizer  # noqa: F401



__all__ = ['set_config', 'get_config', 'describe_config', 'get_all_configs', 'tokenizer_context']

from typing import Any
from collections import defaultdict
from contextlib import contextmanager
from configparser import ConfigParser
from pathlib import Path
from os.path import dirname
import warnings
from rich.console import Console
from rich.table import Table

from .about import __version__


def set_config(config: str, value: Any):  # pylint: disable=unused-argument
    if tuple(map(int, __version__.split('.'))) < (0, 18):  # pylint: disable=no-else-raise
        raise DeprecationWarning(
            "set_config is deprecated with 0.16. Use *_context functions for runtime configuration changes.")
    else:
        raise Exception("set_config should be removed.")


@contextmanager
def tokenizer_context(tokenizer_name, warning=False):
    from .bblock import DocBuilder  # pylint: disable=import-outside-toplevel

    if warning:
        warnings.warn(f"Changing tokenizer to {tokenizer_name}")

    yield DocBuilder(tokenizer=tokenizer_name)


@contextmanager
def config_context(**kwargs):
    from .bblock import DocBuilder  # pylint: disable=import-outside-toplevel

    yield DocBuilder(**kwargs)


@contextmanager
def idf_context(idf_type, warning=False):  # pylint: disable=unused-argument
    from .bblock import DocBuilder  # pylint: disable=import-outside-toplevel

    yield DocBuilder(idf__method=idf_type)


@contextmanager
def tf_context(tf_type, warning=False):  # pylint: disable=unused-argument
    from .bblock import DocBuilder  # pylint: disable=import-outside-toplevel

    yield DocBuilder(tf__method=tf_type)


def get_config(config: str):  # pylint: disable=unused-argument
    if tuple(map(int, __version__.split('.'))) < (0, 18):  # pylint: disable=no-else-raise
        raise DeprecationWarning(
            "get_config is deprecated with 0.16. Use `sadedegel config` command to retrieve configuration")
    else:
        raise Exception("get_config function should be removed.")


def describe_config(config: str, print_desc=False):  # pylint: disable=unused-argument
    if tuple(map(int, __version__.split('.'))) < (0, 18):  # pylint: disable=no-else-raise
        raise DeprecationWarning(
            "get_config is deprecated with 0.16. Use `sadedegel config` command to retrieve configuration")
    else:
        raise Exception("describe_config should be removed.")


def get_all_configs():
    if tuple(map(int, __version__.split('.'))) < (0, 18):  # pylint: disable=no-else-raise
        raise DeprecationWarning(
            "get_config is deprecated with 0.16. Use `sadedegel config` command to retrieve configuration")
    else:
        raise Exception("describe_config should be removed.")


def to_config_dict(kw: dict):
    d = defaultdict(dict)
    for k, v in kw.items():
        if '__' not in k:  # default section
            d['default'][k] = v
        else:
            section, key = k.split('__')

            d[section][key] = v

    return d


def load_config(kwargs: dict = None):
    config = ConfigParser()
    config.read([Path(dirname(__file__)) / 'default.ini', Path("~/.sadedegel/user.ini").expanduser()])

    if kwargs:
        config_dict = to_config_dict(kwargs)
        config.read_dict(config_dict)

    return config


def show_config(config, section=None):
    descriptions = {"default__tokenizer": "Word tokenizer to use",
                    "tf__method": "Method used in term frequency calculation",
                    "tf__double_norm_k": "Smooth parameter used by double norm term frequency method.",
                    "idf__method": "Method used in Inverse Document Frequency calculation"}

    default_config = ConfigParser()
    default_config.read([Path(dirname(__file__)) / 'default.ini'])

    console = Console()

    table = Table(show_header=True, header_style="bold #2070b2")

    table.add_column("section")
    table.add_column("parameter_name")
    table.add_column("current_value")
    table.add_column("default_value")
    table.add_column("description", width=40)

    for sec in config.sections():
        if sec == section or section is None:
            for k in config[sec]:
                if config[sec][k] != default_config[sec][k]:
                    table.add_row(sec, k, f"[orange1]{config[sec][k]}[/orange1]", default_config[sec][k],
                                  descriptions.get(f"{sec}__{k}", ""))
                else:
                    table.add_row(sec, k, f"{config[sec][k]}", default_config[sec][k],
                                  descriptions.get(f"{sec}__{k}", ""))

    console.print(table)



from os.path import expanduser, getsize
from pathlib import Path
from typing import Iterator

import glob
import sys
from collections import defaultdict
import gzip
import json
from itertools import islice

import numpy as np
from sadedegel.config import tokenizer_context

from rich.console import Console
from rich.progress import track

console = Console()

tarballs = ["art_culture.jsonl.gz", "education.jsonl.gz", "horoscope.jsonl.gz", "life_food.jsonl.gz",
            "politics.jsonl.gz", "technology.jsonl.gz", "economics.jsonl.gz", "health.jsonl.gz",
            "life.jsonl.gz", "magazine.jsonl.gz", "sports.jsonl.gz", "travel.jsonl.gz"]

CATEGORIES = [tb.replace('.jsonl.gz', '') for tb in tarballs]

CORPUS_SIZE = 302936

__general_download_message__ = """Ensure that you have properly downloaded extended or tokenization corpus using

            python -m sadedegel.dataset.extended download --access-key xxx --secret-key xxxx
            python -m sadedegel.dataset.tokenization download --access-key xxx --secret-key xxxx

        Unfortunately due to data licensing issues we could not share data publicly. 
        Get in touch with sadedegel team to obtain a download key.

        """

__tokenization_download_message__ = """Ensure that you have properly downloaded tokenization corpus using

            python -m sadedegel.dataset.tokenization download --access-key xxx --secret-key xxxx

        Unfortunately due to data licensing issues we could not share data publicly. 
        Get in touch with sadedegel team to obtain a download key.

        """

__tokenization_version_message__ = "Ensure your dataset is in versioned format."

__tscorpus_validate__ = """It seems that your copy of TSCorpus is [red]corrupted[/red]. For validation run
 
            python -m sadedegel.dataset.tscorpus validate
            
        To download corpus again run
        
            python -m sadedegel.dataset.tscorpus download --access-key xxx --secret-key xxxx
"""


def fail_with_corpus_validation():
    console.log(__tscorpus_validate__)
    sys.exit(1)


def check_directory_structure(path: str) -> bool:
    base_dir = Path(expanduser(path))
    if not base_dir.exists():
        console.log(f"Dataset base directory ([bold red]{base_dir}[/bold red]) does not exist")

    elif not (base_dir / "tscorpus" / "raw").exists() or not (base_dir / "tscorpus" / "tokenized").exists():
        console.log(f"TsCorpus directory ([bold red]{base_dir}/tscorpus[/bold red]) does not exist")

    else:
        return True

    console.log(__general_download_message__)

    return False


def load_tscorpus_raw(data_home="~/.sadedegel_data") -> Iterator[str]:
    if not check_directory_structure(data_home):
        raise Exception("Ts Corpus validation error")

    for tarball in tarballs:
        with gzip.open(Path(expanduser(data_home)) / 'tscorpus' / 'raw' / tarball, 'rt') as fp:
            for line in fp:
                yield json.loads(line)


def load_tokenization_raw(data_home="~/.sadedegel_data") -> Iterator[str]:
    for d in load_tscorpus_raw(data_home):
        yield dict(id=d['id'], text=d['text'])


def load_classification_raw(data_home="~/.sadedegel_data") -> Iterator[str]:
    for d in load_tscorpus_raw(data_home):
        yield dict(id=d['id'], category=CATEGORIES.index(d['category']), text=d['text'])


def load_tokenization_tokenized(data_home="~/.sadedegel_data") -> Iterator[str]:
    if check_directory_structure(data_home):
        for tarball in tarballs:
            with gzip.open(Path(expanduser(data_home)) / 'tscorpus' / 'tokenized' / tarball, 'rt') as fp:
                for line in fp:
                    d = json.loads(line)
                    yield dict(id=d['id'], tokens=d['text'])
    else:
        return None


def raw_stats(data_home: str) -> int:
    sz = 0
    for f in (Path(expanduser(data_home)) / "tscorpus" / "raw").glob("*.gz"):
        sz += getsize(f)
    return sz


def tokenized_stats(data_home: str) -> int:
    sz = 0
    for f in (Path(expanduser(data_home)) / "tscorpus" / "tokenized").glob("*.gz"):
        sz += getsize(f)
    return sz


def check_and_display(data_home="~/.sadedegel_data"):
    if check_directory_structure(data_home):
        return dict(byte=dict(raw=raw_stats(data_home),
                              tokens=tokenized_stats(data_home)))



from math import ceil
from pathlib import Path
from os.path import dirname

from sklearn.linear_model import PassiveAggressiveClassifier
from sklearn.utils import shuffle

from rich.console import Console
from rich.progress import Progress

from joblib import dump, load as jl_load

import numpy as np

from ..dataset.tscorpus import load_classification_raw, CATEGORIES, CORPUS_SIZE
from ..extension.sklearn import TfidfVectorizer, OnlinePipeline

console = Console()


def empty_model():
    return OnlinePipeline(
        [('tfidf', TfidfVectorizer(tf_method='freq', idf_method='smooth')),
         ('pa', PassiveAggressiveClassifier(C=8.23704, average=False))])


def build(max_rows=100_000):
    try:
        import pandas as pd
    except ImportError:
        console.log(("pandas package is not a general sadedegel dependency."
                     " But we do have a dependency on building our prebuilt models"))

    raw = load_classification_raw()
    df = pd.DataFrame.from_records(raw)
    df = shuffle(df)

    if max_rows > 0:
        df = df.sample(max_rows)

    BATCH_SIZE = 1000

    n_split = ceil(len(df) / BATCH_SIZE)
    console.log(f"{n_split} batches of {BATCH_SIZE} instances...")

    batches = np.array_split(df, n_split)

    pipeline = empty_model()

    with Progress() as progress:
        building_task = progress.add_task("[blue]Training a classifier for news categories...",
                                          total=max_rows if max_rows > 0 else CORPUS_SIZE)

        for batch in batches:
            pipeline.partial_fit(batch.text, batch.category, classes=[i for i in range(len(CATEGORIES))])

            progress.update(building_task, advance=len(batch))

    console.log("Model build [green]DONE[/green]")

    model_dir = Path(dirname(__file__)) / 'model'

    model_dir.mkdir(parents=True, exist_ok=True)

    dump(pipeline, (model_dir / 'news_classification.joblib').absolute(), compress=('gzip', 9))


def load(model_name="news_classification"):
    return jl_load(Path(dirname(__file__)) / 'model' / f"{model_name}.joblib")


if __name__ == '__main__':
    build()



from typing import List
import numpy as np

from ..tokenize import Sentences
from ._base import ExtractiveSummarizer


class TFIDFSummarizer(ExtractiveSummarizer):
    tags = ExtractiveSummarizer.tags + ['ml', 'tfidf']

    def __init__(self, normalize=True):
        super().__init__(normalize)

    def _predict(self, sents: List[Sentences]):
        return np.array([sent.tfidf().sum() for sent in sents])



from rich.progress import track
from itertools import islice, tee
import click

from ..dataset.extended import load_extended_sents_corpus
from ..config import tokenizer_context
from .vocabulary import Vocabulary
from ..about import __version__
import warnings


@click.command()
@click.option('--max-doc', help="Maximum documentation in extended corpus", type=int, default=-1)
@click.option('--min-df', help="Mininum document frequenct of a word to be included in", default=3)
@click.option('--word-tokenizer', type=click.Choice(['bert'], case_sensitive=False),
              help="Word tokenizer to be used in building vocabulary.", default='bert')
def build_vocabulary(max_doc, min_df, word_tokenizer):
    """Build vocabulary"""

    if tuple(map(int, __version__.split('.'))) < (0, 18):
        warnings.warn(
            ("sadedegel.bblock.__main__ is deprecated and will be dropped by release 0.18."
             " Please use sadedegel.bblock.cli tokenizer-evaluate"),
            DeprecationWarning,
            stacklevel=2)
    else:
        raise Exception("Remove sadedegel.bblock.__main__ before release.")

    if max_doc > 0:
        corpus = islice(load_extended_sents_corpus(), max_doc)
    else:
        corpus = load_extended_sents_corpus()

    corpus, replica = tee(corpus, 2)
    total = sum(1 for _ in replica)

    vocab = Vocabulary.factory(word_tokenizer)

    click.secho(click.style(f"...Frequency calculation over extended dataset", fg="blue"))

    with tokenizer_context(word_tokenizer) as Doc:
        for i, d in track(enumerate(corpus), total=max_doc if max_doc > 0 else total,
                          description="Building vocabulary..."):
            doc = Doc.from_sentences(d['sentences'])

            for sent in doc:
                for word in sent.tokens:
                    vocab.add_word_to_doc(word, i)

    vocab.build(min_df)
    vocab.save()

    click.secho(click.style(f"Total documents {vocab.document_count}", fg="blue"))
    click.secho(click.style(f"Vocabulary size {len(vocab)} (words occurred more than {min_df} documents)", fg="blue"))


if __name__ == '__main__':
    build_vocabulary()
